function condExpYgivenZ = condExpectation(responseVariable_Y, explanatoryVariable_Z, testStates_Z, kernelWidth)   
% condExpYgivenZ = condExpectation(responseVariable_Y, explanatoryVariable_Z, testStates_Z, kernelWidth)   
%
% empirical estimator of the conditional expectation E[Y|Z] evaluated
% at the states Z = testStates_Z, i.e., an estimate of E[Y|Z=z]
%
% In this context we refer to Y as the responseVariable and to Z as
% the explanatory variable. The conditional expectation E[Y|Z] can be
% understood as a regression function (y = f(z)) of the explanatory
% variable on the response variable, in a least square sense.
%   
% This estimator, however, employes a kernel density regression. In
% particular, it uses a Nadaraya Watson estimator with Gaussian
% kernels.
%
%
% ~~~~~~~~~~ INPUT: ~~~~~~~~~~
%
% responseVariable_Y:   struct with at least the following fields
%                        '.x' - (T,C,N) dimensional data tensor:
%                               leading dimension refers to time;
%                               second dimension to channels, e.g.,
%                               electrodes; third dimension
%                               represents N single trials 
%                        '.t' - temporal indices, length must
%                               correspond to first dimension
%                               of '.x'
%  
% explanatoryVariable_Z: (1,N) row vector of the single trial
%                              observations of the explanatory
%                              variable, the number of trials N
%                              must correspond to the number of
%                              trials of the response variable.
%
% testStates_Z:         (1xS) optional argument
%                             row vector of S instances 'z' of the
%                             explanatory variable, at which the
%                             regression E[Y|Z=z] should be evaluated
%
% kernelWidth:          (1x1) optional argument
%                             positive value reflecting the
%                             standard deviations for the Gaussian
%                             kernels.  
%                                 
% ~~~~~~~~~~ OUTPUT: ~~~~~~~~~~~~ 
%
% condExpYgivenZ:    struct with at least the following fields
%                     '.x' - (T,C,S) dimensional data tensor:
%                            leading dimension refers to time;
%                            second dimension to channels, e.g.,
%                            electrodes; third dimension
%                            represents S instances spezified in
%                            testStates_Z  
%
%                     '.t' - temporal indices, length must
%                            correspond to first dimension of '.x'
%
%                     '.z' - (1xS) dimensional row vector of the
%                            corresponding test states
%  
% Steven Lemm, Berlin, June,15th 2009.
  
  
  % get the dimensionality of the response variable
  [T, nChans, nEvt] = size(responseVariable_Y.x) ;
  
  % get the number of observed and test states of the explantory variable
  numCondStates = length(explanatoryVariable_Z) ;
  numTestStates = length(testStates_Z) ;

  % if kernelWidth is not provided use the default value
  if ~exist('kernelWidth','var') | isempty(kernelWidth),
    kernelWidth = 1.5*(4/3)^0.2 * std(explanatoryVariable_Z)/numCondStates^0.2;
    fprintf('Kernelwidth set to %f\n',kernelWidth);
  end ;

  % create the struct for the result
  condExpYgivenZ = rmfield(responseVariable_Y,{'x'}) ;
  condExpYgivenZ.x = zeros(T, nChans, numTestStates) ;
  condExpYgivenZ.z = testStates_Z;

  
  % loop for all channels 
  for ch = 1: size(responseVariable_Y.x,2),
    counter = 0 ;
    % loop for all sample points
    for sample = responseVariable_Y.t ,
      counter = counter +1 ;

      % apply Nadaraya Watson estimator in order to get estimates
      % for all testStates_Z
      if ~exist('weight_mat','var'),
	
	% weight matrix has NOT been previously computed
	[output, weight_mat] = nadWatCondExp(explanatoryVariable_Z, ...
						     squeeze(responseVariable_Y.x(counter,ch,:))', ...
						     testStates_Z, kernelWidth* ...
						     ones(1, numCondStates) );
      else
	% weight matrix has been previously computed
	[output, weight_mat] = nadWatCondExp(explanatoryVariable_Z, ...
						     squeeze(responseVariable_Y.x(counter,ch,:))', ...
						     testStates_Z, kernelWidth* ...
						     ones(1, numCondStates), weight_mat );
      end % end if ~exist('weight_mat','var'),
      
      % store the results in the 
      condExpYgivenZ.x(counter,ch,:) = output; 
    
    end % end loop for all sample points
  
  end % end loop for all channels
  

  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%         Subfunctions        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  
function [Y_prediction, kernelWeightMatrix] = nadWatCondExp(X_centers, Y_target, X_prediction, kernelWidth, kernelWeightMatrix )
% [output,  kernelWeightMatrix] = nad_wat(X_centers, Y_target, X_prediction, kernelWidth, <kernelWeightMatrix> )
%
% Implementation of a Nadaraya Watson estimator for the univariate
% regression case, using Gaussian kernels
%
% Nadaraya, E. A. (1964). "On Estimating Regression". Theory of
% Probability and its Applications 9 (1):141-142. doi:10.1137/1109020.
% (cf. http://en.wikipedia.org/wiki/Kernel_regression )
% 
%
%~~~~~~~~~~ INPUTs ~~~~~~~~~
% X_centers:          (1xT1) row vector of the centers of the
%                            RBF-kerne, i.e., the given training
%                            data x_centers(i) 
%
% Y_target:           (1xT1) row vector of the corresponding target
%                            values, i.e. Y_target(i) = f(X_centers(i))
%
% X_prediction:       (1xT2) row vector of x-values to make a
%                            prediction at
%
% kernelWidth:        (1xT1) row vector of the kernel widths, i.e,
%                            the standard deviations of the Gaussians
%                            centered at X_centers 
%
% kernelWeightMatrix: (T2xT1) optional argument
%                             kernelWeightMatrix(i,j), i.e., the
%                             contribution of each kernel placed at
%                             X_centers(j) to the final regression
%                             at X_prediction(i)
%  
%~~~~~~~~~~ OUTPUTs ~~~~~~~~~        
% Y_predition         (1xT2) row vector of predicte target values,
%                            i.e., estimated Y_predition(i) = f(X_predition(i)) 
%
% kernelWeightMatrix  (T2xT1) kernelWeightMatrix(i,j), i.e., the
%                             contribution of each kernel placed at
%                             X_centers(j) to the final regression
%                             at X_prediction(i)
%
%
% Steven Lemm, Berlin, June 15th, 2009
 
  
  
if (nargin == 5 | nargin == 4)
     [idim_x, numData_x] = size(X_prediction);
     [idim_y, numData_y] = size(Y_target);
     [idim_comp, numCen] = size(X_centers);
     [idim_sig,numSig] = size(kernelWidth);
     if ( (idim_x ~= idim_comp) | (numCen ~= numSig) | (numCen ~= numData_y) | (idim_sig ~=1) | (idim_y ~=1) )
         error('ERROR in function nadWatCondExp, dimensionalities don''t match\n');
     else
	 %%%%%%%%%%%%%%%%%%%% start calculation of the regression output %%%%%%%%%%%%%%%%%%%%
	 if (nargin == 4)  
	   % estimate the kernelWeightMatrix(i,j), i.e., the
           % contribution of each kernel placed at X_centers(j) to the
           % final regression at X_prediction(i)
	   kernelWeightMatrix =  gaussianKernel(X_prediction, X_centers, kernelWidth');
	 end
	 norm_term = sum(kernelWeightMatrix,2);
	 Y_prediction = (kernelWeightMatrix * Y_target')';
	 Y_prediction = Y_prediction ./ norm_term';

    end
else 
     fprintf('[output, kernelWeightMatrix, norm_term] = nad_wat(X_centers, Y_target, X_prediction, kernelWidth, <kernelWeightMatrix> )\n');
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gaussian kernel
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function out =  gaussianKernel(X_prediction, X_centers, sigma)
% out = gaussianKernel(X_prediction, X_centers, sigma)
% normalized Gaussian kernel function
%
% X_centers       (1xT1) row vector of kernel centers
% sigma           (1xT1) row vector of kernel width
% X_prediction    (1xT2) row vector of x-values
%
% out             (T2xT1)-Matrix
%
% Steven Lemm, Berlin, June 15th, 2009

if (nargin == 3)
     [idim, numData] = size(X_prediction);
     [idim_comp, numCen] = size(X_centers);
     [numSig, dim_sig] = size(sigma);

     if ( (idim ~= idim_comp) | (numCen ~= numSig) | (dim_sig ~=1) )
         fprintf('ERROR in function RBF_KERN\n');
         fprintf('......... dimensionserror in datasizes !!!\n ');
     else
         two_sigma_sq  = (sigma.^2)*2;
         out  = zeros(numData, numCen);
         out  = sqdist(X_prediction', X_centers);
         for i = 1:numCen
             out(:,i) = out(:,i)/ two_sigma_sq(i);
         end
         out = exp(-out);
         for i = 1:numCen
             out(:,i) = out(:,i)/ (sigma(i)*sqrt(2*pi));
         end
     end
else 
  fprintf(' out = rbf_kern(X_prediction, X_centers, sigma\n ');
end
   




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% squared Euclidean distance, as required by the gaussianKernel 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = sqdist(w,p)
%SQDIST Squared distances between vectors.
%	
%	SQDIST(W,P)
%	  W - SxR matrix of rows vectors.
%	  P - RxQ matrix of column vectors.
%	Returns SxQ matrix of squared vectors distances.
%	
%	EXAMPLE: w = [1 2 1];
%	         p = [1; 2.1; 0.9];
%	         n = dist(w,p)
%
% Steven Lemm, Berlin, June 15th, 2009

%#realonly
%#inbounds

[s,r] = size(w);
[r2,q] = size(p);

if r ~= r2, 
	w_Size=[s,r]
	p_Size=[r2,q]
	error('Matrix sizes do not match.'),
end

y = zeros(s,q);

if r == 1
  for i=1:s
    x = w(i,:)'*ones(1,q);
    y(i,:) = abs(x-p).^2;
  end
else
  for i=1:s
    x = w(i,:)'*ones(1,q);
    y(i,:) = sum((x-p).^ 2);
  end
end
