function [trueERD, genCondERD, convCondERD] = example(surrogateDataIdx)
%  [trueERD, genCondERD, convCondERD] = example(surrogateDataIdx)
%
%    provides the basic examples how to calculate state conditional
%    ERD. It reproduces the state conditional ERD analysis of the
%    surrogate data, given in the PLoS publication. It will first call
%    the generating function of surrogate data. Afterwards it
%    calculates state conditional ERD by means of the conventional and
%    the generalized ERD framework. The results are visualized along
%    with the true ERD, which in this particular case is known due to
%    the artificial setting.
% 
%
% ~~~~~INPUT~~~~~
%
%  surrogateDataIdx: natural number indicating the index of the
%                    surrogate dataset, i.e., (1-3) 
%
% ~~~~OUTPUT~~~~
% 
%  trueERD:      struct with at least the following fields
%                     '.x' - (T,C,N) dimensional data tensor of the
%                            estimated generalized conditional ERD:
%                            leading dimension refers to temporal index;
%                            second dimension to channels, e.g.,
%                            electrodes; third dimension
%                            represents N instances corresponding
%                            to states Z=z specified in the field '.z'  
%
%                     '.t' - temporal indices, length corresponds
%                            to first dimension of '.x' 
%
%                     '.z' - (1xN) dimensional row vector of the
%                            corresponding test states Z=z, where
%                            the conditional ERD was evaluated
%
%  genCondERD:   struct as above 
%
%  convCondERD:  struct as above 
%   
%
% Steven Lemm, Berlin, June, 15th, 2009
  


  
% generate the surrogate data according to the settings of the
% examples given in the corresponding PLoS CompBiol publication
[eventRelatedTrials, catchTrials, trueERD] = generateSurrogateData(surrogateDataIdx);


% define the state 'z', where the conditional ERD_{Z=z} should be evaluated
testStates =  eventRelatedTrials.z ; % here we use the same states as in the
				     % training data
				     % however if you would like to
                                     % use a finer grid over the
                                     % Z-domain, i.e, the [0,1] interval instead, just use 
				     % testStates= linspace[0,1,1000];

% the conditonal ERD framework requires a reference interval  
referenceInterval = [-pi, -pi] ; % in the noise free scenario, as in
                                 % the manuscript we used just a
                                 % single instance as reference,
                                 % i.e., t_ref = -pi
				 % alternatively you can specify an
                                 % interval such as [-pi,-2/3*pi]
				 
% estimate the state conditional ERD using the conventional ERD
% framework. Note, this is only done for the purpose of comparing
% the two alternative approaches. For a reliable analysis you
% should of course use the generalized framework.
convCondERD = conventionalConditionalERD(eventRelatedTrials, referenceInterval, testStates);			  

% estimate the generalized condtional ERD 
genCondERD = generalizedConditionalERD(eventRelatedTrials, catchTrials, testStates);


% let's display the results

fprintf('Displaying the results may take some time.\n')
figure ;
set(gcf,'position',[230 642 1265 461]);
ax_hdl = subplot(131);

%plot the true ERD into the left panel
[fig_hdl, ax_hdl, cb_hdl]= displayConditionalERD(trueERD, 'xlabel', 'time','ylabel','explanatory variable Z', 'axes', ax_hdl, 'title','true ERD');   

%plot the generalized ERD into the right panel
ax_hdl = subplot(133);
[fig_hdl, ax_hdl, cb_hdl]= displayConditionalERD(genCondERD, 'xlabel', 'time','ylabel','explanatory variable Z', 'axes', ax_hdl, 'title','generalized ERD');   

%plot the conventional ERD into the central panel
ax_hdl = subplot(132);
[fig_hdl, ax_hdl, cb_hdl]= displayConditionalERD(convCondERD, 'xlabel', 'time','ylabel','explanatory variable Z', 'axes', ax_hdl, 'title','conventional ERD');   


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% subfunctions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% generating function of the surrogate data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [datStim, datCatch, trueERD] = generateSurrogateData(setting)

% number of single trial for each condition (rest and excitation)
numTrials = 500;

% define the temporal index set 
t = -pi:0.1:pi;
T = length(t) ;

% randomly sample the initial phase of sine during rest and
% excitation; one row for each condition (catch and event-related)
xsi = rand(2,numTrials) ;

% randomly sample the explanatory variable during rest and
% excitation; one row for each condition (catch and event-related) 
Z = rand(2,numTrials) ;

% set the row-indices for the two conditions (catch and event-related) 
catchIdx = 1; 
stimIdx  = 2;

% init the data container for the event-related data
datStim.x = zeros(T,1,numTrials) ;
datStim.t = t ;
%datStim.y = ones(1,numTrials) ;
datStim.z = Z(stimIdx,:);

% init the data container for the catch trials
datCatch   = datStim ;
datCatch.z = Z(catchIdx,:);

% init the data container for the true ERD
trueERD   = datStim ;
trueERD.z = Z(stimIdx,:);

%init the multiplicative dampening factor
alpha     = zeros(T,1,numTrials) ;

switch setting
 case 1
  % generate the single trial data for the first example, where
  % solely the ERD depends on Z, but in the presence of a linear trend

  % set the linear trend as in the first example of the corresponding
  % PLoS CompBiol publication
  linTrend  = -t/(3*pi);

  for ev = 1:numTrials,
    % generate the power evelope of an unperturbed catch trial 
    datCatch.x(:,1,ev) = 3/2 + sin(t + 2*pi*xsi(catchIdx,ev))+linTrend;
    
    % determine the dampening, dependent on explanatory variable Z
    alpha(:,1,ev) = dampening(t,Z(stimIdx,ev))';
  
    % generate the unperturbed power envelope for the event-related trial
    datStim.x(:,1,ev) = 3/2 + sin(t + 2*pi*xsi(stimIdx,ev)) +linTrend;

    % attenuate the power envelope by the multiplivative dampening factor
    datStim.x(:,1,ev) = datStim.x(:,1,ev).*alpha(:,1,ev) ;
  end ;
 
 case 2
  % generate the single trial data for the second example, where
  % solely the ongoing activity depends on Z
  for ev = 1:numTrials,
    % generate the single trial during rest 
    datCatch.x(:,1,ev) = 3/2 + sin(t + 2*pi*Z(catchIdx,ev));

    % generate the single trial during stimulation 
    % determine the dampening, dependent on Z
    alpha(:,1,ev) = dampening(t,0)';
    % generate the ongoing activity
    datStim.x(:,1,ev) = 3/2 + sin(t + 2*pi*Z(stimIdx,ev));
    % attenuate the ongoing activity using the dampening factor
    datStim.x(:,1,ev) = datStim.x(:,1,ev).*alpha(:,1,ev) ;
  end ;

 case 3  
  % generate the single trial data for the second example, where
  % solely the ongoing activity depends on Z
  for ev = 1:numTrials,
    % generate the single trial during rest 
    datCatch.x(:,1,ev) = 3/2 + sin(t + 2*pi*Z(catchIdx,ev));
    
    % generate the single trial during stimulation 
    % determine the dampening, dependent on Z
    alpha(:,1,ev) = dampening(t,Z(stimIdx,ev))';
    % generate the ongoing activity
    datStim.x(:,1,ev) = 3/2 + sin(t + 2*pi*Z(stimIdx,ev));
    % attenuate the ongoing activity using the dampening factor
    datStim.x(:,1,ev) = datStim.x(:,1,ev).*alpha(:,1,ev) ;
  end ;

 otherwise  error('The specification of the artificial data set must be either 1, 2, or 3.')
end


% the true ERD corresponds to (multiplicative dampening - 1)
% hence given in percent 
trueERD.x = (alpha-1)*100 ;


function erg = dampening(t,s)
  erg = 1 + (0.75+s/4)*((t-s).^2-1).*(abs(t-s)<=1) ;
  

