function genCondERD = generalizedConditionalERD(eventRelatedTrials, catchTrials, testStates_Z, kernelWidth)   
% genCondERD = generalizedConditionalERD(eventRelatedTrials, catchTrials, testStates_Z, kernelWidth)   
%
% computes the conditional ERD using the generalized framework of a
% dynamic reference, obtained from catch trials.
% 
% ~~~~~~ INPUT ~~~~~
%
% eventRelatedTrials:  struct with at least the following fields
%                        '.x' - (T,C,N1) dimensional data tensor of
%                               the observed event-related trial;
%                               contains the bandpass filtered
%                               signal power. 
%                               Leading dimension refers to time;
%                               second dimension to channels, e.g.,
%                               electrodes; third dimension
%                               represents N1 event-related single trials 
%
%                        '.t' - temporal indices, length must
%                               correspond to first dimension
%                               of '.x'
%
%                        '.z' - (1,N1) dimensional row vector
%                               containing the observed explanatory
%                               states variables 
%
% catchTrials:         struct with at least the following fields
%                      '.x' - (T,C,N2) dimensional data tensor of
%                              the observed catch trials, contains
%                              the bandpass filtered signal power.
%                              Leading dimension refers to time;
%                              second dimension to channels, e.g.,
%                              electrodes; third dimension
%                              represents the N2 catch trials 
%
%                        '.t' - temporal indices, length must
%                               correspond to first dimension
%                               of '.x'
%
%                        '.z' - (1,N2) dimensional row vector
%                               containing the observed explanatory
%                               states variables 
%
% testStates_Z:         (1,S)   optional argument (can be empty)
%                               row vector containing the states of
%                               the explanatory variable Z, where
%                               the empirical estimators should be
%                               evaluated  
% 
% kernelWidth:          (1x1)   optional argument
%                               positive value reflecting the
%                               standard deviations for the Gaussian
%                               kernels.  
% 
% ~~~~ OUTPUT ~~~~~
%
% genCondERD     struct with at least the following fields
%                     '.x' - (T,C,S) dimensional data tensor of the
%                            estimated generalized conditional ERD:
%                            leading dimension refers to time;
%                            second dimension to channels, e.g.,
%                            electrodes; third dimension
%                            represents S instances spezified in
%                            testStates_Z  
%
%                     '.t' - temporal indices, length corresponds
%                            to first dimension of '.x' 
%
%                     '.z' - (1xS) dimensional row vector of the
%                            corresponding test states
%
%
% Steven Lemm, Berlin, June 15th, 2009

  numCondStatesEvent = length(eventRelatedTrials.z) ;
  numCondStatesCatch = length(catchTrials.z) ;

  if ~exist('testStates','var') | isempty(testStates_Z),,
    % set the default values for the test states, i.e., do a linear
    % interpolation between minumum and maximum value of the observed
    % explanatory states
    testStates = linspace(min([eventRelatedTrials.z catchTrials.z]), ...
			  max([eventRelatedTrials.z catchTrials.z]), ...
			  max(numCondStatesEvent,numCondStatesCatch)) ;
  end ;
  
  
  if ~exist('kernelWidth','var') | isempty(kernelWidth),
    % set the default values for the kernel width,
    % (cf. supplementary methods for details)
    kernelWidth = (4/3)^0.2 * std([eventRelatedTrials.z catchTrials.z])/min(numCondStatesEvent,numCondStatesCatch)^0.2;
    fprintf('Kernelwidth set to %f\n',kernelWidth) ;
  end ;

  % get the conditional expected event-related dynamics, evaluated
  % at the test instances 'testStates_Z' 
  condExpYeventGivenZ = condExpectation(eventRelatedTrials, eventRelatedTrials.z, testStates, kernelWidth) ;

  % get the conditional expected unpeturbed dynamics from the catch
  % trials, evaluated at the test instances 'testStates_Z' 
  condExpYcatchGivenZ = condExpectation(catchTrials, catchTrials.z, testStates, kernelWidth) ;
  
  
  % create the struct for the results
  genCondERD = rmfield(condExpYcatchGivenZ,'x') ;

  % estimate the generalized ERD 
  genCondERD.x = (condExpYeventGivenZ.x./condExpYcatchGivenZ.x-1)*100 ;

